module DSGElib

using SymPy
using PyCall
using LinearAlgebra

include("LinRatExp.jl")
using .LinRatExp

include("RiskSensiMethod.jl")
using .RiskSensiMethod

export PolicyFunc
export RiskySS
export ImpResp
export PolicyPlot

Tf = Union{Real,VecOrMat{Float64}}
Ts = Union{Sym{PyObject},VecOrMat{Sym{PyObject}}}
T3 = Union{Real,Vector{Float64},Array{Float64,3}}

vv = Vector{Int64}

function PolicyFunc(F::Vector{Sym{PyObject}},G::Vector{Sym{PyObject}},vars::Tuple,nsss::Tuple,cntrl::vv,state::vv)
    derivs = derivModel(F,G,vars)
    (f,Jf,g,Jg) = derivs(nsss)

    policy = BKstatic(Jf,Jg,g-f,cntrl,state,0)
    return policy
end

function PolicyFunc(F::Vector{Sym{PyObject}},G::Vector{Sym{PyObject}},vars::Tuple,nsss::Tuple,cntrl::vv,state::vv,S::Tf)
    derivs = derivModel2(F,G,vars)
    (f,Jf,Hf,g,Jg) = derivs(nsss)

    policy = RiskSensi(Jf,Hf,Jg,g-f,S,cntrl,state)
    return policy
end

function PolicyFunc(F::Vector{Sym{PyObject}},G::Vector{Sym{PyObject}},vars::Tuple,nsss::Tuple,cntrl::vv,state::vv,Σ::Ts;tol = 1e-12,maxiter = 100,step = 1,showiter = 0)
    derivs = derivModel2(F,G,vars)
    (f,Jf,Hf,g,Jg) = derivs(nsss)

    derivsv = derivVolat(Σ,vars)
    (S,dS) = derivsv(nsss)

    policy = RiskSensiVol(Jf,Hf,Jg,g-f,S,dS,cntrl,state,tolv = tol,maxiterv = maxiter,stepv = step,showiterv = showiter)
    return policy
end

function PolicyFunc(F::Vector{Sym{PyObject}},G::Vector{Sym{PyObject}},vars::Tuple,nsss::Tuple,cntrl::vv,state::vv,S::Tf,Xi::T3)

    derivs = derivModel3(F,G,vars)
    (f,Jf,Hf,H3f,g,Jg) = derivs(nsss)


    policy = RiskSensi3(Jf,Hf,H3f,Jg,g-f,S,Xi,cntrl,state)
    return policy
end

function ImpResp(T::Int64,imp::Tuple,policy::Tuple,cntrl::vv,state::vv)
    Theta = policy[1]
    theta = policy[2]
    Omega = policy[3]
    omega = policy[4]

    ns = length(omega)
    X0 = (I(ns)-Omega)\omega

    n_cntrl = length(theta)
    n_state = length(omega)
    n_vars = n_cntrl+n_state

    
    epsi = zeros(n_vars,T)
    epsi[imp[1],imp[2]] = imp[3]
    variables = zeros(T,n_vars)

    begin
        local X = X0
        for t in 1:T
            X = X.+epsi[state,t];
            Y = Theta*X+theta;
            variables[t,cntrl] = Y';
            variables[t,state] = X';
            X = Omega*X+omega;
        end
    end
    
    return variables
end

function PolicyPlot(StateGrid::Union{Matrix{Float64},Matrix{Int64}},n::Int64,policy::Tuple,f::Union{Function,Nothing} = nothing)
    Theta = policy[1]
    theta = policy[2]
    Omega = policy[3]
    omega = policy[4]

    ns1 = length(omega)
    ns2 = size(StateGrid,1)
    ns1 != ns2 ? error("Number of states is wrong") : nothing
    nc = length(theta)

    C = zeros(ns1,nc,n)
    for i = 1:ns1
        a = StateGrid[i,1]
        b = StateGrid[i,2]
        W = zeros(ns1,n)
        W[i,:] = a:(b-a)/(n-1):b
        C[i,:,:] = Theta*W .+ theta
    end
    return C
end

function RiskySS(F::Vector{Sym{PyObject}},G::Vector{Sym{PyObject}},vars::Tuple,nsss::Tuple,cntrl::vv,state::vv,S::Tf;tol = 1e-12,maxiter = 100,step = 1)
    ns = length(state)
    n = length(cntrl)+ns
    rss = nsss
    SS = zeros(n,1)
    policy = PolicyFunc(F,G,vars,nsss,cntrl,state,S)
    for iter = 2:maxiter
        policy = PolicyFunc(F,G,vars,rss,cntrl,state,S)
        Theta = policy[1]
        theta = policy[2]
        Omega = policy[3]
        omega = policy[4]
        SS[state,:] = (I(ns)-Omega)\omega
        SS[cntrl,:] = Theta*SS[state,:]+theta
        if maximum(abs.(SS)) < tol
            println("rss converged in $(iter) iterations")
            break
        else
            rss = rss.+step .*(SS...,)
            iter == maxiter ? println("rss didn't converge") : true
        end
    end
    return rss,policy
end

function RiskySS(F::Vector{Sym{PyObject}},G::Vector{Sym{PyObject}},vars::Tuple,nsss::Tuple,cntrl::vv,state::vv,Σ::Ts;tol = 1e-12,maxiter = 100,step = 1,tol2 = 1e-12,maxiter2 = 100,step2 = 1)
    ns = length(state)
    n = length(cntrl)+ns
    rss = nsss
    SS = zeros(n,1)
    policy = PolicyFunc(F,G,vars,nsss,cntrl,state,Σ,tol = tol2,maxiter = maxiter2,step = step2)
    for iter = 2:maxiter
        policy = PolicyFunc(F,G,vars,rss,cntrl,state,Σ,tol = tol2,maxiter = maxiter2,step = step2)
        Theta = policy[1]
        theta = policy[2]
        Omega = policy[3]
        omega = policy[4]
        SS[state,:] = (I(ns)-Omega)\omega
        SS[cntrl,:] = Theta*SS[state,:]+theta
        if maximum(abs.(SS)) < tol
            println("rss converged in $(iter) iterations")
            break
        else
            rss = rss.+step .*(SS...,)
            iter == maxiter ? println("rss didn't converge") : true
        end
    end
    return rss,policy
end

function RiskySS(F::Vector{Sym{PyObject}},G::Vector{Sym{PyObject}},vars::Tuple,nsss::Tuple,cntrl::vv,state::vv,S::Tf,Xi::T3;tol = 1e-12,maxiter = 100,step = 1)
    ns = length(state)
    n = length(cntrl)+ns
    rss = nsss
    SS = zeros(n,1)
    policy = PolicyFunc(F,G,vars,nsss,cntrl,state,S,Xi)
    for iter = 2:maxiter
        policy = PolicyFunc(F,G,vars,rss,cntrl,state,S,Xi)
        Theta = policy[1]
        theta = policy[2]
        Omega = policy[3]
        omega = policy[4]
        SS[state,:] = (I(ns)-Omega)\omega
        SS[cntrl,:] = Theta*SS[state,:]+theta
        if maximum(abs.(SS)) < tol
            println("rss converged in $(iter) iterations")
            break
        else
            rss = rss.+step .*(SS...,)
            iter == maxiter ? println("rss didn't converge") : true
        end
    end
    return rss,policy
end

end