% This program computes a risk sensitive linear approximation to the
% price-dividend ratio of the asset pricing model of  Burnside (1998, JEDC) 
% used in ``Risk sensitive linear approximations,''
% by Gustavo Solorzano Andrade and Juan Carlos Parra-Alvarez. 
%
% Important: Add folder utils to the path
%
% User inputs:
% Input 1: Parameter values 
% Input 2: Symbolic definition of control and state variables
% Input 3: Define the Euler system
% Input 4: Deterministic steady state (DSS)
%
% (c) Gustavo Solorzano Andrade and Juan Carlos Parra-Alvarez
% Date February 2, 2024.

%% ====================================================================== %
%                             Housekeeping                                %
% ----------------------------------------------------------------------- %
close all
clear
clc

%% ====================================================================== %
%                    Add working directory and toolbox                    %
% ----------------------------------------------------------------------- %
addpath('../utils/');

%% =======================================================================%
%                                 Input 1:                                %
%                                Parameters                               %
% ----------------------------------------------------------------------- %
beta   = 0.95;
gam    = 2.5;
z_star = 0.0179;
phi_z  = -0.139;
x_star = 0.0012;

% Parametric constraint for convergence of analytical solution (Thm 1, pp. 333) 
cons = beta*exp((1-gam)*z_star+0.5*((1-gam)/(1-phi_z))^2*x_star);
if cons >= 1
    error('Analytical price-dividend ratio does not converge: check parameter values')
end

% Algorithm options
randn('seed',305);
shock       = 1; %z stn dev (1), random (2)
Tsim        = 20; 
plots       = 1;
maxiter     = 100000;
crit        = sqrt(eps);
order       = 2;

%% =======================================================================%
%                                 Input 2:                                %
%                       Define control and states (y,x)                   %
%-------------------------------------------------------------------------%
syms lq z
h       = [lq z];
cntrl   = [1];
state   = [2];
n_state = length(state);
n       = length(h);

%% =======================================================================%
%                                 Input 3:                                %
%                           Model's definition                            %
% ----------------------------------------------------------------------- %
F = [beta*exp((1-gam)*z)*(1+exp(lq));
     z];

G = [exp(lq);
     (1-phi_z)*z_star+phi_z*z];
 
%% =======================================================================%
%                                 Input 4:                                %
%                     Deterministic steady state (DSS)                    %
%-------------------------------------------------------------------------%
var_ss = solve(F-G,h);
lq_ss   = var_ss.lq; 
z_ss   = var_ss.z;  

%% ====================================================================== %
%                    Build model's solution approximation                 %
% ----------------------------------------------------------------------- %
[Jf,Hf,H3f,Jg,Hg,H3g] = derivModel(F,G,h,order);

Sig      = sym(zeros(n_state));
Sig(1,1) = x_star;
DSig     = derivVolat(Sig,h);

lq = eval(lq_ss);
z  = eval(z_ss);
for adjust = 1:maxiter
  nF  = eval(F);
  nG  = eval(G);
  nJf = eval(Jf);
  nJg = eval(Jg);
  if order ~= 1
    nHf = eval(Hf);
  end
  
  nSig  = eval(Sig);
  nDSig = eval(DSig); 
  
  if order == 1
      [Theta,theta,Omega,omega] = ParrSolv(nJf,0,nJg,nG-nF,nSig,nDSig,cntrl,state);
  else
      [Theta,theta,Omega,omega] = ParrSolv(nJf,nHf,nJg,nG-nF,nSig,nDSig,cntrl,state);
  end

  SS(state,1) = (eye(n_state)-Omega)\omega;
  SS(cntrl,1) = Theta*SS(state,1)+theta;
  if max(abs(SS))<crit
    break
  else
    lq = lq+SS(1);
    z = z+SS(2);
  end
end

%% ====================================================================== %
%                        Stochastic Steady state                          %
% ----------------------------------------------------------------------- %
var_sss = [lq;z];

%% ====================================================================== %
%                           Model simulation                              %
% ----------------------------------------------------------------------- %
if shock == 1
  eps_z = zeros(Tsim,1);
  eps_z(2) = 1;  
elseif shock == 2
  eps_z = randn(Tsim,1);
else
  warning('No shock selected');
  eps_z = zeros(Tsim,1);
end
v = chol(nSig)';

tt = zeros(Tsim,1);
tt(1,1) = 0;
for i = 2:Tsim
    tt(i,1) = tt(i-1,1) + 1;
end

%% ====================================================================== %
%                          Policy functions                               %
% ----------------------------------------------------------------------- %
Model_I_Analytical;
zz(1) = z+sqrt(x_star)*eps_z(1);
X0    = 0;
for t = 1:Tsim
  % Approximated solution  
  X0 = X0+v*eps_z(t);
  Y0 = Theta*X0+theta;
  variables(t,cntrl) = Y0';
  variables(t,state) = X0';
  X0 = Omega*X0+omega;  
  
  % Closed form solution
  zx     = [zz(t)];
  qq(t)  = f_analytical_Burnside(zx);  
  if t < Tsim    
    zz(t+1) = (1-phi_z)*z_star+phi_z*zz(t)+sqrt(x_star)*eps_z(t+1);
  end
end

%% ====================================================================== %
%                                 Plots                                   %
% ----------------------------------------------------------------------- %
if plots == 1
    fig = figure(1);    
    subplot(2,1,1)
    plot(tt(1:end),exp(variables(:,1)+lq),'-','linewidth',2);    
    hold on
    plot(tt(1:end),qq,'linewidth',2)
    hold off
    title('Price/dividend ratio','interpreter','latex');
    grid on;
    axis([0 Tsim -inf inf]);
    set(gca,'fontsize',15, 'box', 'off')    
    set(gcf,'color','w')  
    
    subplot(2,1,2)
    plot(tt(1:end),variables(:,2)+z,'-','linewidth',2);   
    hold on
    plot(tt(1:end),zz,'linewidth',2)
    hold off
    title('Dividend growth','interpreter','latex');
    grid on;
    axis([0 Tsim -inf inf]);
    h2 = legend('Risk Sensitive linearization','True solution','Location','NorthEast');
    set(h2, 'interpreter','latex','fontsize',15)
    legend('boxoff')   
    set(gca,'fontsize',15, 'box', 'off')    
    set(gcf,'color','w')      

    %% Plot policy function in Figure 1 of the paper
    grayColor = [.7 .7 .7];
    z_ss = z_star;
    q_ss = (beta*exp((1-gam)*z_star))/(1-beta*exp((1-gam)*z_star));
    n_points = 50;
    z_range = linspace(-0.25,0.25,n_points);
    z_range = sort([z_range,z_star]);
    
    % Allocation of memory
    lq_order2 = NaN(length(z_range),1);
    z_order2  = NaN(length(z_range),1);

    % Approximated stochastic steady state
    lq_order2_sss = var_sss(1);
    z_order2_sss = var_sss(2);

    for i = 1:length(z_range)
        lq_order2(i,1) = Theta*[z_range(i)-z_order2_sss]+theta;
        z_order2(i,1) = Omega(1,:)*[z_range(i)-z_order2_sss]+omega(1,1);
    end
    q_order2 = exp(lq_order2 + lq_order2_sss);
    z_order2 = z_order2 + z_order2_sss;

    q_True = NaN(length(z_range),1);
    for i = 1:length(z_range)
        q_True(i,1) = f_analytical_Burnside(z_range(i));
    end

    fig = figure(2);    
    plot(z_range-z_ss,q_order2,'r','linewidth',2.5);
    hold on
    plot(z_range-z_ss,q_True,'b--','linewidth',1.5);
    hold on
    plot(0,q_ss,'Marker','o','MarkerSize',8,'MarkerEdgeColor','k','LineWidth',1.5)    
    hold off
    ylabel('Model I','interpreter','latex','Fontsize',16)
    xlabel('$z_t-\bar{z}$','interpreter','latex','Fontsize',16)
    grid on;
    set(gca,'fontsize',16,'box','off')
    set(gcf, 'color','w')
    axis([-inf inf -inf inf])
    h2 = legend('Risk-Sensitive','True','Location','SouthEast');
    set(h2,'FontSize',14,'interpreter','latex')
    legend('boxoff') 
end

% end-of-file
