% This program computes a risk sensitive linear approximation to the
% price-dividend ratio of the asset pricing model of Burnside (1998, JEDC) 
% with exponential shocks used in ``Risk sensitive linear approximations,''
% by Gustavo Solorzano Andrade and Juan Carlos Parra-Alvarez. 
%
% Important: Add folder utils to the path
%
% User inputs:
% Input 1: Parameter values 
% Input 2: Symbolic definition of control and state variables
% Input 3: Define the Euler system
% Input 4: Deterministic steady state (DSS)
%
% (c) Gustavo Solorzano Andrade and Juan Carlos Parra-Alvarez
% Date February 2, 2024.


%% ====================================================================== %
%                             Housekeeping                                %
% ----------------------------------------------------------------------- %
close all
clear
clc

%% ====================================================================== %
%                    Add working directory and toolbox                    %
% ----------------------------------------------------------------------- %
addpath('../utils/');

%% =======================================================================%
%                                 Input 1:                                %
%                                Parameters                               %
% ----------------------------------------------------------------------- %
beta   = 0.95;
gam    = 2.5;
z_star = 0.0179;
phi_z  = -0.139;
x      = sqrt(0.0012);

% Algorithm options
rand('seed',305);
shock       = 1; %z stn dev (1), random (2)
Tsim        = 20; 
plots       = 1;
maxiter     = 100000;
crit        = sqrt(eps);
order       = 3;

%% =======================================================================%
%                                 Input 2:                                %
%                       Define control and states (y,x)                   %
%-------------------------------------------------------------------------%
syms lq z
h       = [lq z];
cntrl   = [1];
state   = [2];
n_state = length(state);
n       = length(h);

%% =======================================================================%
%                                 Input 3:                                %
%                           Model's definition                            %
% ----------------------------------------------------------------------- %
F = [beta*exp((1-gam)*z)*(1+exp(lq));
     z];

G = [exp(lq);
     (1-phi_z)*z_star+phi_z*z];

%% =======================================================================%
%                                 Input 4:                                %
%                     Deterministic steady state (DSS)                    %
%-------------------------------------------------------------------------%
var_ss = solve(F-G,h);
lq_ss  = var_ss.lq; 
z_ss   = var_ss.z; 

%% ====================================================================== %
%                    Build model's solution approximation                 %
% ----------------------------------------------------------------------- %
[Jf,Hf,H3f,Jg,Hg,H3g] = derivModel(F,G,h,order);

nSig      = zeros(n_state);
nSig(1,1) = x^2;
nXi_symb  = [2*x^3];

lq = eval(lq_ss);
z  = eval(z_ss);

for adjust = 1:maxiter
  nF   = eval(F);
  nG   = eval(G);
  nJf  = eval(Jf);
  nJg  = eval(Jg);
  if order == 2
      nHf  = eval(Hf);
  elseif order > 2
      nHf  = eval(Hf);
      nH3f = eval(H3f);
  end
  
  %nSig      = eval(Sig);
  %nXi_symb  = eval(Xi_symb);
  
  Xi        = zeros(n_state,n_state,n_state);
  Xi(1,1,1) = nXi_symb;
  
  if order == 1
      [Theta,theta,Omega,omega] = BKstaticEq(nJf,nJg,nG-nF,cntrl,state,0);
  elseif order == 2
      [Theta,theta,Omega,omega] = ParrSol(nJf,nHf,nJg,nG-nF,nSig,cntrl,state);
  elseif order == 3
      [Theta,theta,Omega,omega] = ParrSol3(nJf,nHf,nH3f,nJg,nG-nF,nSig,Xi,cntrl,state);      
  end

  SS(state,1) = (eye(n_state)-Omega)\omega;
  SS(cntrl,1) = Theta*SS(state,1)+theta;
  if max(abs(SS))<crit
    break
  else
    lq = lq+SS(1);
    z  = z+SS(2);
  end
end

%% ====================================================================== %
%                        Stochastic Steady state                          %
% ----------------------------------------------------------------------- %
var_sss = [lq;z];

%% ====================================================================== %
%                           Model simulation                              %
% ----------------------------------------------------------------------- %
if shock == 1
  eps_z = zeros(Tsim,1);
  eps_z(5) = -log(1-0.95)-1;
elseif shock == 2
  eps_z = zeros(Tsim,1);
  eps_z(5) = -log(1-0.05)-1;
elseif shock == 3
  eps_z = -log(1-rand(Tsim,1))-1;
else
  warning('No shock selected');
  eps_z = zeros(Tsim,1);
end

%% ====================================================================== %
%                          Policy functions                               %
% ----------------------------------------------------------------------- %
Model_IV_Analytical;
zz(1) = z+x*eps_z(1);
X0    = [0];
for t = 1:Tsim
  % Approximated solution
  X0 = X0+x*[eps_z(t)];
  Y0 = Theta*X0+theta;
  variables(t,cntrl) = Y0';
  variables(t,state) = X0';  
  X0 = Omega*X0+omega;
  
  % Closed form solution
  qq(t) = f_analytical_de_groot_exp(zz(t));
  if t < Tsim
    zz(t+1) = (1-phi_z)*z_star+phi_z*zz(t)+x*eps_z(t+1);
  end
end

%% ====================================================================== %
%                                 Plots                                   %
% ----------------------------------------------------------------------- %
if plots == 1
    fig = figure(1);
    setmyfig(fig)
    subplot(1,2,1)
    plot(exp(variables(:,1)+lq),'-','linewidth',2);
    hold on
    plot(qq,'linewidth',2)
    hold off
    title('Price/dividend ratio','interpreter','latex');
    grid on;
    set(gca,'fontsize',15)
    
    subplot(1,2,2)
    plot(variables(:,2)+z,'-','linewidth',2);
    hold on
    plot(zz,'linewidth',2)
    hold off
    title('Dividend growth','interpreter','latex');
    grid on;
    set(gca,'fontsize',15)    
    h2 = legend(['Risk Sensitive linearization (order ',num2str(order),')'],'True solution','Location','SouthEast');
    set(h2, 'interpreter','latex','fontsize',15)
    legend('boxoff')  

    %% Plot policy function in Figure 1 of the paper
    
    grayColor = [.7 .7 .7];
    z_ss = z_star;
    q_ss = (beta*exp((1-gam)*z_star))/(1-beta*exp((1-gam)*z_star));
    n_points = 50;
    z_range = linspace(-0.25,0.25,n_points);
    z_range = sort([z_range,z_star]);    

    % Allocation of memory
    lq_order3 = NaN(length(z_range),1);
    z_order3  = NaN(length(z_range),1);

    % Approximated stochastic steady state
    lq_order3_sss = var_sss(1);
    z_order3_sss = var_sss(2);

    for i = 1:length(z_range)
        lq_order3(i,1) = Theta*[z_range(i)-z_order3_sss]+theta;
        z_order3(i,1) = Omega(1,:)*[z_range(i)-z_order3_sss]+omega(1,1);
    end
    q_order3 = exp(lq_order3 + lq_order3_sss);
    z_order3 = z_order3 + z_order3_sss;

    Model_IV_Analytical;
    q_True = NaN(length(z_range),1);
    for i = 1:length(z_range)   
        q_True(i,1) = f_analytical_de_groot_exp(z_range(i));
    end
    
    fig = figure(2);
    plot(z_range-z_ss,q_order3,'r','linewidth',2);
    hold on
    plot(z_range-z_ss,q_True,'b--','linewidth',2);
    hold on
    plot(0,q_ss,'Marker','o','MarkerSize',10,'MarkerEdgeColor','k','LineWidth',2)
    hold off
    ylabel('Model IV','interpreter','latex','Fontsize',16)
    xlabel('$z_t-\bar{z}$','interpreter','latex','Fontsize',16)
    grid on;
    set(gca,'fontsize',16,'box','off')
    set(gcf, 'color','w')
    axis([-inf inf -inf inf])
    h2 = legend('Risk-Sensitive Linearization (order 3)','True','Location','SouthEast');
    set(h2,'FontSize',16,'interpreter','latex')
    legend('boxoff') 

    axes('position',[0.20 0.60 0.25 0.25]);
    box on
    your_index = -0.15 < z_range  & z_range  < -0.13;    
    plot(z_range(your_index)-z_ss,q_order3(your_index),'r','linewidth',2);
    hold on
    plot(z_range(your_index)-z_ss,q_True(your_index),'b--','linewidth',2);
    axis tight
    grid on
end

% end-of-file